unit TrekCurve;

{**********************************************************}
{                                                          }
{  TTrekCurve Unit                                         }
{  Copyright 1999 Workshell Software.                     }
{							   }
{  Version 1.0						   }
{                                                          }
{                                                          }
{  Web      -> http://www.workshell.uni.cc/                }
{  E - Mail -> trekvcl@kinsella.u-net.com                  }
{                                                          }
{**********************************************************}

interface

{$I TrekVer.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
 TCurveLayout = (clTopLeft, clBottomLeft, clTopRight, clBottomRight);

type
  TTrekCurve = class(TCustomControl)
  private
    { Private declarations }
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FLayout: TCurveLayout;
    FColor: TColor;
    FBorderColor: TColor;
    FBackgroundColor: TColor;
    procedure SetLayout(Value: TCurveLayout);
    procedure SetColor(Value: TColor);
    procedure SetBorderColor(Value: TColor);
    procedure SetBackgroundColor(Value: TColor);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    { Protected declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  public
    { Public declarations }
  published
    { Published declarations }
    property Layout: TCurveLayout read FLayout write SetLayout;
    property Color: TColor read FColor write SetColor;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property BackgroundColor: TColor read FBackgroundColor write SetBackgroundColor;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property Align;
    property Enabled;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
   {$IFDEF D4CB4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
   {$ENDIF}
  end;

procedure Register;

implementation

constructor TTrekCurve.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
  ControlStyle := [csCaptureMouse, csClickEvents, csOpaque, csDoubleClicks, csReplicatable];
  Width := 153;
  Height := 65;
  Color := $00CF9030;
  BorderColor := $00CF9030;
  BackgroundColor := clBlack;
  Layout := clTopLeft;
end;

destructor TTrekCurve.Destroy;
begin
 inherited Destroy;
end;

procedure TTrekCurve.Paint;
var
 dy, d2: Integer;
 PenWidth: Integer;
 bx1, bx2: Integer;
 r2: Integer;
 vWidth: Integer;
 Bitmap: TBitMap;
 OldColor: TColor;
begin
 Bitmap := TBitmap.Create;
  with Bitmap.Canvas do
   begin
    Bitmap.Height:=Height;
    Bitmap.Width:= Width;
    Pen.Color := FBorderColor;
    Brush.Color := FColor;
    PenWidth := Pen.Width;
    dy := Height div 2;
    bx1 := Penwidth;
    OldColor := Brush.Color;
    Brush.Color := FBackgroundColor;
    FillRect(Rect(0, 0, Width, Height));
    Brush.Color := OldColor;
    if FLayout = clTopLeft then
     begin
      bx2 := Height - PenWidth;
      d2 := dy - PenWidth;
      r2 := d2 div 2;
      vWidth := Round(Width * 0.56);
      vWidth := vWidth + PenWidth;
      Arc(bx1,bx1,bx2,bx2,dy,bx1,bx1,dy);
      Arc(vWidth, dy, vWidth + d2, dy + d2, r2 + vWidth, dy, vWidth,dy + r2);
      MoveTo(bx1, dy);
      LineTo(bx1, bx2);
      LineTo(vWidth,bx2);
      LineTo(vWidth, dy + r2 - 2);
      MoveTo(dy, bx1);
      LineTo(Width - PenWidth, bx1);
      LineTo(Width - PenWidth, dy);
      LineTo(vWidth + r2, dy);
      FloodFill(PenWidth + 5, bx2 - 10, Pen.Color, fsBorder);
     end;
    if FLayout = clBottomLeft then
     begin
      bx2:= Height - PenWidth;
      d2 := dy - PenWidth;
      r2 := d2 div 2;
      vWidth := Round(Width * 0.56);
      vWidth := vWidth + penwidth;
      Arc(bx1, bx1, bx2, bx2, bx1, dy, dy, bx2);
      Arc(vWidth, dy - d2, vWidth + d2, dy, vWidth, dy - r2, vWidth + r2, dy);
      MoveTo(bx1, dy);
      LineTo(bx1, bx1);
      LineTo(vWidth, bx1);
      LineTo(vWidth, dy - r2 + 2);
      MoveTo(dy, bx2);
      LineTo(Width - PenWidth, bx2);
      LineTo(Width - PenWidth, dy);
      LineTo(vWidth + r2 - 2, dy);
      FloodFill(bx1 + PenWidth, bx1 + PenWidth, Pen.Color, fsBorder);
     end;
    if FLayout = clTopRight then
     begin
      bx2 := Height - PenWidth;
      d2 := dy - PenWidth;
      r2 := d2 div 2;
      vWidth := Round(Width * 0.56);
      vWidth := vWidth + penwidth;
      Arc(Width - bx2, bx1, Width - PenWidth, bx2, Width - PenWidth, dy, Width - dy, bx1);
      Arc(Width - vWidth - d2 , dy, Width - vWidth, dy + d2, Width - vWidth, dy + r2, Width - vWidth - r2, dy);
      MoveTo(Width - PenWidth, dy);
      LineTo(Width - PenWidth, bx2);
      LineTo(Width - vWidth, bx2);
      LineTo(Width - vWidth, dy + r2 - 2);
      MoveTo(Width - dy, bx1);
      LineTo(bx1, bx1);
      LineTo(bx1, dy);
      LineTo(Width - vWidth - r2 + 2, dy);
      FloodFill(PenWidth + 5, PenWidth + 5, Pen.Color, fsBorder);
     end;
    if FLayout = clBottomRight then
     begin
      bx2 := Height - PenWidth;
      d2 := dy - PenWidth;
      r2 := d2 div 2;
      vWidth := Round(Width * 0.56);
      vWidth:= vWidth + PenWidth;
      Arc(Width - bx2, bx1, Width - PenWidth, bx2, Width - dy, bx2, Width - PenWidth, dy);
      Arc(Width - vWidth - d2, dy - d2, Width - vWidth, dy, Width - vWidth - r2, dy, Width - vWidth, dy - r2);
      MoveTo(Width - PenWidth, dy);
      LineTo(Width - PenWidth, bx1);
      LineTo(Width - vWidth, bx1);
      LineTo(Width - vWidth, dy - r2 + 2);
      MoveTo(Width - dy, bx2);
      LineTo(bx1, bx2);
      LineTo(bx1, dy);
      LineTo(Width - vWidth - r2 + 2, dy);
      FloodFill(PenWidth + 5, bx2 - 10, Pen.Color, fsBorder);
     end;
   end;
Canvas.CopyMode := cmSrcCopy;
Canvas.Draw(0,0, Bitmap);
Bitmap.Free;
end;

procedure TTrekCurve.SetLayout(Value: TCurveLayout);
begin
 FLayout := Value;
 Repaint;
end;

procedure TTrekCurve.SetColor(Value: TColor);
begin
 FColor := Value;
 Repaint;
end;

procedure TTrekCurve.SetBorderColor(Value: TColor);
begin
 FBorderColor := Value;
 Repaint;
end;

procedure TTrekCurve.SetBackgroundColor(Value: TColor);
begin
 FBackgroundColor := Value;
 Repaint;
end;

procedure TTrekCurve.CMMouseEnter(var Message: TMessage);
begin
if not (csDesigning in ComponentState) and (Enabled = True) then
 begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
 end;
end;

procedure TTrekCurve.CMMouseLeave(var Message: TMessage);
begin
if not (csDesigning in ComponentState) and (Enabled = True) then
 begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
 end;
end;

procedure Register;
begin
 RegisterComponents('StarTrek',[TTrekCurve]);
end;

end.
