unit TrekPanel;

{**********************************************************}
{                                                          }
{  TTrekPanel Unit                                         }
{  Copyright 1999 Workshell Software.                     }
{							   }
{  Version 1.0						   }
{                                                          }
{                                                          }
{  Web      -> http://www.workshell.uni.cc/                }
{  E - Mail -> trekvcl@kinsella.u-net.com                  }
{                                                          }
{**********************************************************}

interface

{$I TrekVer.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TTrekPanel = class(TCustomControl)
  private
    { Private declarations }
    FAlignment: TAlignment;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FColor: TColor;
    FBorderColor: TColor;
    procedure SetLayout(Value: TAlignment);
    procedure SetColor(Value: TColor);
    procedure SetBorderColor(Value: TColor);
    procedure SetCaption(var Message: TMessage); message CM_TEXTCHANGED;
    procedure SetFont(var Message: TMessage);     message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    { Protected declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  public
    { Public declarations }
  published
    { Published declarations }
    property Alignment: TAlignment read FAlignment write SetLayout;
    property Color: TColor read FColor write SetColor;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property Align;
    property Caption;
    property Enabled;
    property Font;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
   {$IFDEF D4CB4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
   {$ENDIF}
  end;

procedure Register;

implementation

constructor TTrekPanel.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
  Width := 75;
  Height := 25;
  Color := clBlack;
  BorderColor := $0057D1FF;
  Alignment := taCenter;
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
  csSetCaption, csOpaque, csDoubleClicks, csReplicatable];
end;

destructor TTrekPanel.Destroy;
begin
 inherited Destroy;
end;

procedure TTrekPanel.Paint;
var
 X, Y: Integer;
begin
 with Canvas do
  begin
   with ClientRect do
    begin
     Pen.Color := FBorderColor;
     Brush.Color := FColor;
     Rectangle(Left,Top,Right,Bottom);
     Canvas.Font.Assign(Self.Font);
     if FAlignment = taCenter then
      begin
       X := Width div 2 - TextWidth(Caption) div 2;
       Y := Height div 2 - TextHeight(Caption) div 2;
       TextOut(X, Y, Caption);
      end;
     if FAlignment = taLeftJustify then
      begin
       X := Left;
       Y := Height div 2 - TextHeight(Caption) div 2;
       TextOut(X, Y, Caption);
      end;
     if FAlignment = taRightJustify then
      begin
       X := Width - TextWidth(Caption);
       Y := Height div 2 - TextHeight(Caption) div 2;
       TextOut(X, Y, Caption);
      end;
    end;
  end;
end;

procedure TTrekPanel.SetLayout(Value: TAlignment);
begin
 FAlignment := Value;
 Repaint;
end;

procedure TTrekPanel.SetColor(Value: TColor);
begin
 FColor := Value;
 Repaint;
end;

procedure TTrekPanel.SetBorderColor(Value: TColor);
begin
 FBorderColor := Value;
 Repaint;
end;

procedure TTrekPanel.SetCaption(var Message: TMessage);
begin
 Repaint;
end;

procedure TTrekPanel.SetFont(var Message: TMessage);
begin
 inherited;
 Repaint;
end;

procedure TTrekPanel.CMMouseEnter(var Message: TMessage);
begin
if not (csDesigning in ComponentState) and (Enabled = True) then
 begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
 end;
end;

procedure TTrekPanel.CMMouseLeave(var Message: TMessage);
begin
if not (csDesigning in ComponentState) and (Enabled = True) then
 begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
 end;
end;

procedure Register;
begin
 RegisterComponents('StarTrek',[TTrekPanel]);
end;

end.
